<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;

class RelCancelPorOperadorService {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function fetchReport($conditions) {
	    try{ 
  	     $conditions = self::prepareParameters($conditions);
  	     $params = array(
  	         ':CDFILIAL' => $conditions['CDFILIAL'], 
  	         ':TCDFILIAL'=> $conditions['TCDFILIAL'], 
  	         ':CDLOJA'   => $conditions['CDLOJA'],
  	         ':TCDLOJA'  => $conditions['TCDLOJA'],
  	         ':DTINI'    => $conditions['DATAINI'],
  	         ':DTFIN'    => $conditions['DATAFIN'],
  	         ':NRORG'    => $conditions['NRORG']
  	     );
  	    $rows = $this->entityManager->getConnection()->fetchAll(self::QRY_RELATORIO, $params);
  	    return $rows;
	    } catch(\Exception $e){
	      throw($e);
	    }
	}
	
	private function prepareParameters($conditions){
        if(!array_key_exists('CDFILIAL', $conditions)) {
            $conditions['CDFILIAL'] = NULL;
            $conditions['TCDFILIAL'] = 'T';
        } else {
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $conditions['TCDFILIAL'] = NULL;
        }
        if(!array_key_exists('CDLOJA', $conditions)) {
            $conditions['CDLOJA'] = NULL;
            $conditions['TCDLOJA'] = 'T';
        } else {
            $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
            $conditions['TCDLOJA'] = NULL;
        }
        if(array_key_exists('DTENTRVENDA', $conditions)) {
            $conditions['DATAINI'] = $conditions['DTENTRVENDA'][0];
            $conditions['DATAFIN'] = $conditions['DTENTRVENDA'][1];
            unset($conditions['DTENTRVENDA']);
        }
        return $conditions;
    }

	
    const QRY_RELATORIO = "
        SELECT T.CDOPERADOR, T.NMOPERADOR, T.CDOPERADOR||' '||T.NMOPERADOR AS LABELOPERADOR, T.DTINI, T.DTFIN, ROUND(NVL(SUM(T.QTDCUPON), 0)) AS QTDCUPON, NVL(SUM(T.VRQTDE), 0) AS VRQTDE,
         NVL(SUM(T.VRITEN), 0) AS VRITEN, ROUND(NVL(SUM(T.QTDITEN), 0)) AS QTDITEN, (NVL(SUM(T.VRITEN), 0) +  NVL(SUM(T.VRQTDE), 0)) AS VRTOTAL,
         T.CDFILIAL, T.CDLOJA, T.CDFILIAL||' - '||T.NMFILIAL AS LABELFILIAL, T.NRORG
        FROM 
          (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.DTINI, OP.DTFIN, OP.CDLOJA, OP.CDFILIAL, COUNT(DISTINCT VN.CDFILIAL||VN.CDCAIXA||VN.NRSEQVENDA ) AS QTDCUPON,
             SUM(TRUNC(ITV.QTPRODVEND * (NVL(ITV.VRUNITVEND,0) + NVL(ITV.VRUNITVENDCL,0)),2) - NVL(ITV.VRDESITVEND,0) + NVL(ITV.VRACRITVEND,0)) AS VRQTDE,
             NULL AS QTDITEN, NULL AS VRITEN, OP.NMFILIAL, OP.NRORG
          FROM VENDA VN 
          JOIN ITEMVENDA ITV
           ON VN.NRSEQVENDA = ITV.NRSEQVENDA
            AND VN.CDFILIAL = ITV.CDFILIAL
            AND VN.CDCAIXA = ITV.CDCAIXA
            AND VN.NRORG = ITV.NRORG
          LEFT JOIN ( SELECT CUPOM.CDFILIAL,CUPOM.CDLOJA, CUPOM.DTINI, CUPOM.DTFIN, CUPOM.CDOPERADOR, CUPOM.NMOPERADOR, CUPOM.NMFILIAL, CUPOM.NRORG
                      FROM (
                            SELECT DISTINCT  F.CDFILIAL, F.NMFILIAL, L.CDLOJA, :DTINI AS DTINI, :DTFIN AS DTFIN, O.CDOPERADOR, O.NMOPERADOR,
                             V.NRORG
                            FROM VENDA V
                            JOIN FILIAL F 
                             ON V.CDFILIAL = F.CDFILIAL 
                              AND V.NRORG  = F.NRORG
                            JOIN LOJA L 
                             ON V.CDFILIAL = L.CDFILIAL
                              AND V.CDLOJA = L.CDLOJA
                              AND V.NRORG  = L.NRORG            
                            JOIN OPERADOR O
                             ON V.CDOPERADOR = O.CDOPERADOR
                              AND V.NRORG = O.NRORG
                             WHERE V.IDSITUVENDA = 'C'
                              AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                              AND ((INSTR(:CDLOJA  , V.CDFILIAL||V.CDLOJA) <> 0) OR (:TCDLOJA   = 'T'))
                              AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                              AND V.NRORG = :NRORG
                            GROUP BY F.CDFILIAL, L.CDLOJA, :DTINI, :DTFIN, O.CDOPERADOR, O.NMOPERADOR, F.NMFILIAL, V.NRORG
                      ) CUPOM
                      ORDER BY CUPOM.CDFILIAL, CUPOM.CDLOJA, CUPOM.CDOPERADOR, CUPOM.NMFILIAL
                    ) OP
           ON  VN.CDFILIAL = OP.CDFILIAL   
            AND VN.CDLOJA = OP.CDLOJA
            AND VN.CDOPERADOR = OP.CDOPERADOR
            AND VN.NRORG = OP.NRORG
          WHERE 
            VN.DTENTRVENDA BETWEEN OP.DTINI AND OP.DTFIN
            AND VN.IDSITUVENDA = 'C'
          GROUP BY OP.CDOPERADOR, OP.NMOPERADOR, OP.DTINI, OP.DTFIN, OP.CDLOJA, OP.CDFILIAL, OP.NMFILIAL, OP.NRORG
          
          UNION 
          
          SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.DTINI, OP.DTFIN, OP.CDLOJA, OP.CDFILIAL,NULL AS QTDCUPON, NULL AS VRQTDE, SUM(ITC.QTPRODVENDC) AS QTDITEN,
                 SUM(TRUNC(ITC.QTPRODVENDC * (NVL(ITC.VRUNITVENDC,0) + NVL(ITC.VRUNITVENCLC,0)),2) - NVL(ITC.VRDESITVENDC,0) + NVL(ITC.VRACRITVENDC,0)) AS VRITEN,
                 OP.NMFILIAL, OP.NRORG
          FROM VENDA VN 
          JOIN ITVENDACAN ITC
           ON VN.NRSEQVENDA = ITC.NRSEQVENDA
             AND VN.CDFILIAL = ITC.CDFILIAL
             AND VN.CDCAIXA = ITC.CDCAIXA
             AND VN.NRORG = ITC.NRORG
          LEFT JOIN (SELECT CUPOM.CDFILIAL,CUPOM.CDLOJA, CUPOM.DTINI, CUPOM.DTFIN, CUPOM.CDOPERADOR, CUPOM.NMOPERADOR, CUPOM.NMFILIAL, CUPOM.NRORG
                      FROM (
                            SELECT DISTINCT  F.CDFILIAL, L.CDLOJA, :DTINI AS DTINI, :DTFIN AS DTFIN, O.CDOPERADOR, O.NMOPERADOR,
                             F.NMFILIAL, V.NRORG
                            FROM VENDA V
                            JOIN FILIAL F 
                             ON V.CDFILIAL = F.CDFILIAL 
                              AND V.NRORG = F.NRORG
                            JOIN LOJA L 
                             ON V.CDFILIAL = L.CDFILIAL
                              AND V.CDLOJA = L.CDLOJA
                              AND V.NRORG= L.NRORG
                            JOIN ITVENDACAN IT
                                ON V.NRSEQVENDA = IT.NRSEQVENDA
                                    AND V.CDFILIAL = IT.CDFILIAL
                                    AND V.CDCAIXA = IT.CDCAIXA
                                    AND V.NRORG = IT.NRORG
                            JOIN OPERADOR O
                             ON NVL(IT.CDOPERCANC, V.CDOPERADOR) = O.CDOPERADOR
                             AND V.NRORG = O.NRORG
                              AND ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                              AND ((INSTR(:CDLOJA  , V.CDFILIAL||V.CDLOJA) <> 0) OR (:TCDLOJA   = 'T'))
                              AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                              AND V.NRORG = :NRORG
                            GROUP BY F.CDFILIAL, L.CDLOJA, :DTINI, :DTFIN, O.CDOPERADOR, O.NMOPERADOR, F.NMFILIAL, V.NRORG
                      ) CUPOM
                      ORDER BY CUPOM.CDFILIAL, CUPOM.CDLOJA, CUPOM.CDOPERADOR, CUPOM.NMFILIAL, CUPOM.NRORG
                    ) OP
           ON  VN.CDFILIAL = OP.CDFILIAL   
            AND VN.CDLOJA = OP.CDLOJA
            AND NVL(ITC.CDOPERCANC, VN.CDOPERADOR) = OP.CDOPERADOR
            AND VN.NRORG = OP.NRORG
          WHERE 
            VN.DTENTRVENDA BETWEEN OP.DTINI AND OP.DTFIN
          GROUP BY OP.CDOPERADOR, OP.NMOPERADOR, OP.DTINI, OP.DTFIN, OP.CDLOJA, OP.CDFILIAL, OP.NMFILIAL, OP.NRORG
        ) T 
        GROUP BY T.CDOPERADOR, T.NMOPERADOR, T.CDOPERADOR||' '||T.NMOPERADOR, T.DTINI, T.DTFIN, T.CDFILIAL, T.CDLOJA, T.CDFILIAL||' - '||T.NMFILIAL, T.NRORG
        ORDER BY T.CDOPERADOR
    ";   
}